@@ -3,17 +3,19 @@ use strict;
 use warnings;
 use warnings::register;
 use Carp;
+use Config;
 use Exporter        ();
-use Fcntl           qw< O_WRONLY >;
 use File::Basename;
 use POSIX           qw< strftime setlocale LC_TIME >;
 use Socket          qw< :all >;
 require 5.005;
 
 
+*import = \&Exporter::import;
+
+
 {   no strict 'vars';
-    $VERSION = '0.31';
-    @ISA     = qw< Exporter >;
+    $VERSION = '0.34';
 
     %EXPORT_TAGS = (
         standard => [qw(openlog syslog closelog setlogmask)],
@@ -73,6 +75,29 @@ require 5.005;
 }
 
 
+#
+# Constants
+#
+use constant HAVE_GETPROTOBYNAME     => $Config::Config{d_getpbyname};
+use constant HAVE_GETPROTOBYNUMBER   => $Config::Config{d_getpbynumber};
+use constant HAVE_SETLOCALE          => $Config::Config{d_setlocale};
+use constant HAVE_IPPROTO_TCP        => defined &Socket::IPPROTO_TCP ? 1 : 0;
+use constant HAVE_IPPROTO_UDP        => defined &Socket::IPPROTO_UDP ? 1 : 0;
+use constant HAVE_TCP_NODELAY        => defined &Socket::TCP_NODELAY ? 1 : 0;
+
+use constant SOCKET_IPPROTO_TCP =>
+      HAVE_IPPROTO_TCP      ? Socket::IPPROTO_TCP
+    : HAVE_GETPROTOBYNAME   ? scalar getprotobyname("tcp")
+    : 6;
+
+use constant SOCKET_IPPROTO_UDP =>
+      HAVE_IPPROTO_UDP      ? Socket::IPPROTO_UDP
+    : HAVE_GETPROTOBYNAME   ? scalar getprotobyname("udp")
+    : 17;
+
+use constant SOCKET_TCP_NODELAY => HAVE_TCP_NODELAY ? Socket::TCP_NODELAY : 1;
+
+
 # 
 # Public variables
 # 
@@ -91,7 +116,7 @@ my $connected       = 0;        # flag to indicate if we're connected or not
 my $syslog_send;                # coderef of the function used to send messages
 my $syslog_path     = undef;    # syslog path for "stream" and "unix" mechanisms
 my $syslog_xobj     = undef;    # if defined, holds the external object used to send messages
-my $transmit_ok     = 0;        # flag to indicate if the last message was transmited
+my $transmit_ok     = 0;        # flag to indicate if the last message was transmitted
 my $sock_port       = undef;    # socket port
 my $sock_timeout    = 0;        # socket timeout, see below
 my $current_proto   = undef;    # current mechanism used to transmit messages
@@ -147,6 +172,14 @@ my @fallbackMethods = ();
 
 $sock_timeout = 0.001 if $^O =~ /darwin|gnukfreebsd/;
 
+
+# Perl 5.6.0's warnings.pm doesn't have warnings::warnif()
+if (not defined &warnings::warnif) {
+    *warnings::warnif = sub {
+        goto &warnings::warn if warnings::enabled(__PACKAGE__)
+    }
+}
+
 # coderef for a nicer handling of errors
 my $err_sub = $options{nofatal} ? \&warnings::warnif : \&croak;
 
@@ -235,7 +268,9 @@ my %mechanism = (
         check   => sub {
             return 1 if defined $sock_port;
 
-            if (getservbyname('syslog', 'tcp') || getservbyname('syslogng', 'tcp')) {
+            if (eval { local $SIG{__DIE__};
+                getservbyname('syslog','tcp') || getservbyname('syslogng','tcp')
+            }) {
                 $host = $syslog_path;
                 return 1
             }
@@ -249,7 +284,7 @@ my %mechanism = (
         check   => sub {
             return 1 if defined $sock_port;
 
-            if (getservbyname('syslog', 'udp')) {
+            if (eval { local $SIG{__DIE__}; getservbyname('syslog', 'udp') }) {
                 $host = $syslog_path;
                 return 1
             }
@@ -329,8 +364,8 @@ sub setlogsock {
             $found = 1;
         }
         else {
-            warnings::warnif "setlogsock(): type='$sock_type': "
-                           . $mechanism{$sock_type}{err_msg};
+            warnings::warnif("setlogsock(): type='$sock_type': "
+                           . $mechanism{$sock_type}{err_msg});
         }
     }
 
@@ -341,8 +376,7 @@ sub setlogsock {
 }
 
 sub syslog {
-    my $priority = shift;
-    my $mask = shift;
+    my ($priority, $mask, @args) = @_;
     my ($message, $buf);
     my (@words, $num, $numpri, $numfac, $sum);
     my $failed = undef;
@@ -361,6 +395,7 @@ sub syslog {
     if ($priority =~ /^\d+$/) {
         $numpri = LOG_PRI($priority);
         $numfac = LOG_FAC($priority) << 3;
+        undef $numfac if $numfac == 0;  # no facility given => use default
     }
     elsif ($priority =~ /^\w+/) {
         # Allow "level" or "level|facility".
@@ -409,13 +444,14 @@ sub syslog {
 
     if ($mask =~ /%m/) {
         # escape percent signs for sprintf()
-        $error =~ s/%/%%/g if @_;
+        $error =~ s/%/%%/g if @args;
         # replace %m with $error, if preceded by an even number of percent signs
         $mask =~ s/(?<!%)((?:%%)*)%m/$1$error/g;
     }
 
-    $mask .= "\n" unless $mask =~ /\n$/;
-    $message = @_ ? sprintf($mask, @_) : $mask;
+    # add (or not) a newline
+    $mask .= "\n" if !$options{noeol} and rindex($mask, "\n") == -1;
+    $message = @args ? sprintf($mask, @args) : $mask;
 
     if ($current_proto eq 'native') {
         $buf = $message;
@@ -428,17 +464,27 @@ sub syslog {
         $whoami .= "[$$]" if $options{pid};
 
         $sum = $numpri + $numfac;
-        my $oldlocale = setlocale(LC_TIME);
-        setlocale(LC_TIME, 'C');
-        my $timestamp = strftime "%b %d %H:%M:%S", localtime;
-        setlocale(LC_TIME, $oldlocale);
+
+        my $oldlocale;
+        if (HAVE_SETLOCALE) {
+            $oldlocale = setlocale(LC_TIME);
+            setlocale(LC_TIME, 'C');
+        }
+
+        # %e format isn't available on all systems (Win32, cf. CPAN RT #69310)
+        my $day = strftime "%e", localtime;
+
+        if (index($day, "%") == 0) {
+            $day = strftime "%d", localtime;
+            $day =~ s/^0/ /;
+        }
+
+        my $timestamp = strftime "%b $day %H:%M:%S", localtime;
+        setlocale(LC_TIME, $oldlocale) if HAVE_SETLOCALE;
 
         # construct the stream that will be transmitted
         $buf = "<$sum>$timestamp $whoami: $message";
 
-        # add (or not) a newline
-        $buf .= "\n" if !$options{noeol} and rindex($buf, "\n") == -1;
-
         # add (or not) a NUL character
         $buf .= "\0" if !$options{nonul};
     }
@@ -448,7 +494,8 @@ sub syslog {
     if ($options{perror} and $current_proto ne 'native') {
         my $whoami = $ident;
         $whoami .= "[$$]" if $options{pid};
-        print STDERR "$whoami: $message\n";
+        print STDERR "$whoami: $message";
+        print STDERR "\n" if rindex($message, "\n") == -1;
     }
 
     # it's possible that we'll get an error from sending
@@ -512,11 +559,11 @@ sub _syslog_send_console {
     } else {
         if (open(CONS, ">/dev/console")) {
 	    my $ret = print CONS $buf . "\r";  # XXX: should this be \x0A ?
-	    POSIX::_exit $ret if defined $pid;
+	    POSIX::_exit($ret) if defined $pid;
 	    close CONS;
 	}
 
-	POSIX::_exit if defined $pid;
+	POSIX::_exit(0) if defined $pid;
     }
 }
 
@@ -617,14 +664,9 @@ sub connect_log {
 sub connect_tcp {
     my ($errs) = @_;
 
-    my $proto = getprotobyname('tcp');
-    if (!defined $proto) {
-	push @$errs, "getprotobyname failed for tcp";
-	return 0;
-    }
-
-    my $port = $sock_port || getservbyname('syslog', 'tcp');
-    $port = getservbyname('syslogng', 'tcp') unless defined $port;
+    my $port = $sock_port
+            || eval { local $SIG{__DIE__}; getservbyname('syslog',   'tcp') }
+            || eval { local $SIG{__DIE__}; getservbyname('syslogng', 'tcp') };
     if (!defined $port) {
 	push @$errs, "getservbyname failed for syslog/tcp and syslogng/tcp";
 	return 0;
@@ -642,16 +684,14 @@ sub connect_tcp {
     }
     $addr = sockaddr_in($port, $addr);
 
-    if (!socket(SYSLOG, AF_INET, SOCK_STREAM, $proto)) {
+    if (!socket(SYSLOG, AF_INET, SOCK_STREAM, SOCKET_IPPROTO_TCP)) {
 	push @$errs, "tcp socket: $!";
 	return 0;
     }
 
     setsockopt(SYSLOG, SOL_SOCKET, SO_KEEPALIVE, 1);
-    if (silent_eval { IPPROTO_TCP() }) {
-        # These constants don't exist in 5.005. They were added in 1999
-        setsockopt(SYSLOG, IPPROTO_TCP(), TCP_NODELAY(), 1);
-    }
+    setsockopt(SYSLOG, SOCKET_IPPROTO_TCP, SOCKET_TCP_NODELAY, 1);
+
     if (!connect(SYSLOG, $addr)) {
 	push @$errs, "tcp connect: $!";
 	return 0;
@@ -665,13 +705,8 @@ sub connect_tcp {
 sub connect_udp {
     my ($errs) = @_;
 
-    my $proto = getprotobyname('udp');
-    if (!defined $proto) {
-	push @$errs, "getprotobyname failed for udp";
-	return 0;
-    }
-
-    my $port = $sock_port || getservbyname('syslog', 'udp');
+    my $port = $sock_port
+            || eval { local $SIG{__DIE__}; getservbyname('syslog', 'udp') };
     if (!defined $port) {
 	push @$errs, "getservbyname failed for syslog/udp";
 	return 0;
@@ -689,7 +724,7 @@ sub connect_udp {
     }
     $addr = sockaddr_in($port, $addr);
 
-    if (!socket(SYSLOG, AF_INET, SOCK_DGRAM, $proto)) {
+    if (!socket(SYSLOG, AF_INET, SOCK_DGRAM, SOCKET_IPPROTO_UDP)) {
 	push @$errs, "udp socket: $!";
 	return 0;
     }
@@ -716,15 +751,21 @@ sub connect_stream {
     # might want syslog_path to be variable based on syslog.h (if only
     # it were in there!)
     $syslog_path = '/dev/conslog' unless defined $syslog_path; 
+
     if (!-w $syslog_path) {
 	push @$errs, "stream $syslog_path is not writable";
 	return 0;
     }
-    if (!sysopen(SYSLOG, $syslog_path, O_WRONLY, 0400)) {
+
+    require Fcntl;
+
+    if (!sysopen(SYSLOG, $syslog_path, Fcntl::O_WRONLY(), 0400)) {
 	push @$errs, "stream can't open $syslog_path: $!";
 	return 0;
     }
+
     $syslog_send = \&_syslog_send_stream;
+
     return 1;
 }
 
@@ -893,7 +934,7 @@ Sys::Syslog - Perl interface to the UNIX syslog(3) calls
 
 =head1 VERSION
 
-This is the documentation of version 0.31
+This is the documentation of version 0.34
 
 =head1 SYNOPSIS
 
@@ -1002,7 +1043,7 @@ process, so this option has no effect on Linux.)
 =item *
 
 C<perror> - Write the message to standard error output as well to the
-system log (added in C<Sys::Syslo> 0.22).
+system log (added in C<Sys::Syslog> 0.22).
 
 =item *
 
@@ -1246,7 +1287,7 @@ Now that the "native" mechanism is supported by C<Sys::Syslog> and selected
 by default, the use of the C<setlogsock()> function is discouraged because 
 other mechanisms are less portable across operating systems.  Authors of 
 modules and programs that use this function, especially its cargo-cult form 
-C<setlogsock("unix")>, are advised to remove any occurence of it unless they 
+C<setlogsock("unix")>, are advised to remove any occurrence of it unless they 
 specifically want to use a given mechanism (like TCP or UDP to connect to 
 a remote host).
 
@@ -1550,6 +1591,14 @@ Perl and C<Sys::Syslog> versions.
 
 =head1 SEE ALSO
 
+=head2 Other modules
+
+L<Log::Log4perl> - Perl implementation of the Log4j API
+
+L<Log::Dispatch> - Dispatches messages to one or more outputs
+
+L<Log::Report> - Report a problem, with exceptions and language support
+
 =head2 Manual Pages
 
 L<syslog(3)>
@@ -1646,7 +1695,19 @@ You can find documentation for this module with the perldoc command.
 
 You can also look for information at:
 
-=over 4
+=over
+
+=item * Perl Documentation
+
+L<http://perldoc.perl.org/Sys/Syslog.html>
+
+=item * MetaCPAN
+
+L<https://metacpan.org/module/Sys::Syslog>
+
+=item * Search CPAN
+
+L<http://search.cpan.org/dist/Sys-Syslog/>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
@@ -1660,20 +1721,11 @@ L<http://cpanratings.perl.org/d/Sys-Syslog>
 
 L<http://rt.cpan.org/Dist/Display.html?Queue=Sys-Syslog>
 
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/Sys-Syslog/>
-
-=item * MetaCPAN
-
-L<https://metacpan.org/module/Sys::Syslog>
-
-=item * Perl Documentation
-
-L<http://perldoc.perl.org/Sys/Syslog.html>
-
 =back
 
+The source code is available on Git Hub:
+L<https://github.com/maddingue/Sys-Syslog/>
+
 
 =head1 COPYRIGHT
 
